@@ -7,7 +7,7 @@
 //! The main trick is keeping in-place linked list of freed indexes for reuse.
 //!
 //! Serde is supported. If you need pre-computed length at serialization time
-//! (for example, for bincode), use serde_ser_len feature.
+//! (for example, for bincode), use `serde_ser_len` feature.
 
 
 #[cfg(test)]
@@ -23,17 +23,20 @@ use std::ops::{Index, IndexMut};
 use std::slice;
 use std::vec;
 use std::fmt;
+use std::clone::Clone;
+
+const SENTINEL: usize = usize::MAX;
 
 #[derive(Clone)]
 enum Entry<V> {
     Empty(usize),
-    Occupied(V)
+    Occupied(V),
 }
 
 impl<V> Entry<V> {
     fn is_empty(&self) -> bool {
-        match self {
-            &Entry::Empty(_) => true,
+        match *self {
+            Entry::Empty(_) => true,
             _ => false,
         }
     }
@@ -59,7 +62,7 @@ impl<V> Entry<V> {
 #[derive(Clone)]
 pub struct CompactMap<V> {
     data: Vec<Entry<V>>,
-    free_head: usize
+    free_head: usize,
 }
 
 impl<V> CompactMap<V> {
@@ -74,7 +77,7 @@ impl<V> CompactMap<V> {
     pub fn new() -> CompactMap<V> {
         CompactMap {
             data: vec![],
-            free_head: usize::MAX
+            free_head: SENTINEL,
         }
     }
 
@@ -90,7 +93,7 @@ impl<V> CompactMap<V> {
     pub fn with_capacity(capacity: usize) -> Self {
         CompactMap {
             data: Vec::with_capacity(capacity),
-            free_head: usize::MAX
+            free_head: SENTINEL,
         }
     }
 
@@ -106,27 +109,40 @@ impl<V> CompactMap<V> {
     pub fn reserve(&mut self, len: usize) {
         self.data.reserve(len);
     }
-    
+
     /// Reserves capacity for `CompactMap`'s underlying vector.
     /// If you just cleared M elements from the map and want to insert N
     /// more elements, you'll probably need to reserve N-M elements.
     pub fn reserve_exact(&mut self, len: usize) {
         self.data.reserve_exact(len);
     }
-    
-    // TODO: keys
-    // TODO: values
-    // TODO: values_mut
-    // TODO: append
-    // TODO: clear
+
     // TODO: entry
-    
+    // TODO: DoubleEndedIterator
+
+    /// Clears the map, removing all key-value pairs.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use compactmap::CompactMap;
+    ///
+    /// let mut a = CompactMap::new();
+    /// a.insert("a");
+    /// a.clear();
+    /// assert!(a.is_empty_slow());
+    /// ```
+    pub fn clear(&mut self) {
+        self.free_head = SENTINEL;
+        self.data.clear();
+    }
+
     /// Iterating the map to check if it is empty.
     /// O(n) where n is historical maximum element count.
     pub fn is_empty_slow(&self) -> bool {
-        return self.len_slow() == 0
+        self.len_slow() == 0
     }
-    
+
     /// Inserts a value into the map. The map generates and returns ID of
     /// the inserted element.
     ///
@@ -148,7 +164,7 @@ impl<V> CompactMap<V> {
     pub fn insert(&mut self, v: V) -> usize {
         let head = self.free_head;
         let entry = Entry::Occupied(v);
-        if head == usize::MAX {
+        if head == SENTINEL {
             self.data.push(entry);
             self.data.len() - 1
         } else {
@@ -157,11 +173,11 @@ impl<V> CompactMap<V> {
                     self.free_head = next;
                     head
                 }
-                Entry::Occupied(_) => unreachable!()
+                Entry::Occupied(_) => unreachable!(),
             }
         }
     }
-    
+
     /// Removes a key from the map, returning the value at the key if the key
     /// was previously in the map.
     /// ```
@@ -174,13 +190,13 @@ impl<V> CompactMap<V> {
     /// ```
     pub fn remove(&mut self, i: usize) -> Option<V> {
         if i >= self.data.len() {
-            return None
+            return None;
         }
         if let Entry::Empty(_) = self.data[i] {
             // Early return to avoid further wrong mem::replace
-            return None
+            return None;
         }
-        
+
         let empty_entry = Entry::Empty(self.free_head);
         if let Entry::Occupied(v) = mem::replace(&mut self.data[i], empty_entry) {
             if i == self.data.len() - 1 {
@@ -189,25 +205,27 @@ impl<V> CompactMap<V> {
                 self.free_head = i;
             }
             Some(v)
-        } else { unreachable!(); }
+        } else {
+            unreachable!();
+        }
     }
-    
+
     /// Returns a reference to the value corresponding to the key.
     pub fn get(&self, i: usize) -> Option<&V> {
         self.data.get(i).and_then(|entry| match *entry {
             Entry::Empty(_) => None,
-            Entry::Occupied(ref v) => Some(v)
+            Entry::Occupied(ref v) => Some(v),
         })
     }
-    
+
     /// Returns a mutable reference to the value corresponding to the key.
     pub fn get_mut(&mut self, i: usize) -> Option<&mut V> {
         self.data.get_mut(i).and_then(|entry| match *entry {
             Entry::Empty(_) => None,
-            Entry::Occupied(ref mut v) => Some(v)
+            Entry::Occupied(ref mut v) => Some(v),
         })
     }
-    
+
     /// Returns an iterator visiting all key-value pairs in unspecified order.
     /// The iterator's element type is `(usize, &'r V)`.
     ///
@@ -226,30 +244,55 @@ impl<V> CompactMap<V> {
     ///     println!("{}: {}", key, value);
     /// }
     /// ```
-    pub fn iter<'a>(&'a self) -> Iter<'a, V> {
-        Iter { iter: self.data.iter(), counter: 0 }
+    pub fn iter(&self) -> Iter<V> {
+        Iter {
+            iter: self.data.iter(),
+            counter: 0,
+        }
     }
-    
+
     /// Returns an iterator visiting all key-value pairs in unspecified order,
     /// with mutable references to the values.
     /// The iterator's element type is `(usize, &'r mut V)`
-    pub fn iter_mut<'a>(&'a mut self) -> IterMut<'a, V> {
-        IterMut { iter: self.data.iter_mut(), counter: 0 }
+    pub fn iter_mut(&mut self) -> IterMut<V> {
+        IterMut {
+            iter: self.data.iter_mut(),
+            counter: 0,
+        }
     }
-    
+
     /// Returns an iterator visiting all key-value pairs in unspecified order,
     /// the keys, consuming the original `CompactMap`.
     /// The iterator's element type is `(usize, V)`.
     pub fn into_iter(self) -> IntoIter<V> {
-        IntoIter { iter: self.data.into_iter(), counter: 0 }
+        IntoIter {
+            iter: self.data.into_iter(),
+            counter: 0,
+        }
     }
-    
+
+    /// Returns an iterator visiting all keys in some order.
+    /// The iterator's element type is `usize`.
+    pub fn keys(&self) -> Keys<V> {
+        Keys { iter: self.iter() }
+    }
+    /// Returns an iterator visiting all values in ascending order of the keys.
+    /// The iterator's element type is `&'r V`.
+    pub fn values(&self) -> Values<V> {
+        Values { iter: self.iter() }
+    }
+    /// Returns an iterator visiting all values in ascending order of the keys.
+    /// The iterator's element type is `&'r mut V`.
+    pub fn values_mut(&mut self) -> ValuesMut<V> {
+        ValuesMut { iter_mut: self.iter_mut() }
+    }
+
     /// Iterates the map to get number of elements.
     /// O(n) where n is historical maximum element count.
     pub fn len_slow(&self) -> usize {
         self.iter().count()
     }
-    
+
     /// Trims the `CompactMap` of any excess capacity.
     ///
     /// Rescans the whole map to reindex empty slots. O(n).
@@ -283,9 +326,42 @@ impl<V> CompactMap<V> {
         self.reindex();
     }
 
-    
+    /// Returns an iterator visiting all key-value pairs in ascending order of
+    /// the keys, emptying (but not consuming) the original `CompactMap`.
+    /// The iterator's element type is `(usize, V)`. Keeps the allocated memory for reuse.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use compactmap::CompactMap;
+    ///
+    /// let mut map = CompactMap::new();
+    /// map.insert("a");
+    /// map.insert("b");
+    /// map.insert("c");
+    /// map.remove(1);
+    ///
+    /// let vec: Vec<(usize, &str)> = map.drain().collect();
+    ///
+    /// assert_eq!(vec, [(0, "a"), (2, "c")]);
+    /// assert!(map.is_empty_slow());
+    /// assert!(map.capacity() > 0);
+    /// ```
+    pub fn drain(&mut self) -> Drain<V> {
+        fn filter<A>((i, v): (usize, Entry<A>)) -> Option<(usize, A)> {
+            match v {
+                Entry::Empty(_) => None,
+                Entry::Occupied(x) => Some((i,x)),
+            }
+        }
+        let filter: fn((usize, Entry<V>)) -> Option<(usize, V)> = filter; // coerce to fn ptr
+
+        self.free_head = SENTINEL;
+        Drain { iter: self.data.drain(..).enumerate().filter_map(filter) }
+    }
+
     fn reindex(&mut self) {
-        self.free_head = usize::MAX;
+        self.free_head = SENTINEL;
         for i in 0..self.data.len() {
             if let Entry::Empty(ref mut head) = self.data[i] {
                 *head = self.free_head;
@@ -302,8 +378,14 @@ impl<V> Default for CompactMap<V> {
 }
 
 
-impl<V> Hash for CompactMap<V> where V: Hash {
-    fn hash<H>(&self, state: &mut H) where H: Hasher {
+impl<V> Hash for CompactMap<V>
+where
+    V: Hash,
+{
+    fn hash<H>(&self, state: &mut H)
+    where
+        H: Hasher,
+    {
         for i in 0..(self.data.len()) {
             if let Entry::Occupied(ref j) = self.data[i] {
                 state.write_usize(i);
@@ -352,11 +434,14 @@ macro_rules! iterate_for_ord_and_eq {
 
 impl<V: Eq> Eq for CompactMap<V> {}
 
-// We are greater then them iif { { we have i'th slot 
+// We are greater then them iif { { we have i'th slot
 // filled in and they don't } or { data in i'th slot compares
-// "greater" to our data } } and filledness status and contained data 
+// "greater" to our data } } and filledness status and contained data
 // prior to i is the same.
-impl<V> PartialOrd<CompactMap<V>> for CompactMap<V> where V: PartialOrd<V> {
+impl<V> PartialOrd<CompactMap<V>> for CompactMap<V>
+where
+    V: PartialOrd<V>,
+{
     fn partial_cmp(&self, other: &CompactMap<V>) -> Option<Ordering> {
         iterate_for_ord_and_eq!(self, other,
                                 Some(Ordering::Greater), Some(Ordering::Less),
@@ -372,7 +457,10 @@ impl<V> PartialOrd<CompactMap<V>> for CompactMap<V> where V: PartialOrd<V> {
     }
 }
 
-impl<V> Ord for CompactMap<V> where V: Ord {
+impl<V> Ord for CompactMap<V>
+where
+    V: Ord,
+{
     fn cmp(&self, other: &CompactMap<V>) -> Ordering {
         iterate_for_ord_and_eq!(self, other,
                                 Ordering::Greater, Ordering::Less,
@@ -389,35 +477,74 @@ impl<V> Ord for CompactMap<V> where V: Ord {
 }
 
 impl<V> FromIterator<V> for CompactMap<V> {
-    fn from_iter<I>(iter: I) -> CompactMap<V> where I: IntoIterator<Item=V> {
+    fn from_iter<I>(iter: I) -> CompactMap<V>
+    where
+        I: IntoIterator<Item = V>,
+    {
         let mut c = CompactMap::new();
-        // TODO size hint here maybe
-        for i in iter {
+        
+        let it = iter.into_iter();
+        
+        reasonable_reserve(&mut c.data, it.size_hint());
+        
+        for i in it {
             c.insert(i);
         }
         c
     }
 }
 
-impl<'a, V> FromIterator<&'a V> for CompactMap<V> where V : Copy {
+impl<'a, V> FromIterator<&'a V> for CompactMap<V>
+where
+    V: Copy,
+{
     #[allow(map_clone)]
-    fn from_iter<I>(iter: I) -> CompactMap<V> where I: IntoIterator<Item=&'a V> {
+    fn from_iter<I>(iter: I) -> CompactMap<V>
+    where
+        I: IntoIterator<Item = &'a V>,
+    {
         FromIterator::<V>::from_iter(iter.into_iter().map(|&value| value))
     }
 }
 
+fn reasonable_reserve<T>(v: &mut Vec<T>, (rmin, mbrmax) : (usize, Option<usize>)) {
+    use std::cmp::{min,max};
+    if let Some(rmax) = mbrmax {
+        if rmin == rmax {
+            v.reserve_exact(rmin);
+        } else {
+            let reasonable_reserve = max(rmin, min(rmax, 4096));
+            v.reserve(reasonable_reserve);
+        }
+    } else {
+        v.reserve(rmin);
+    }
+}
+
 impl<V> Extend<V> for CompactMap<V> {
-    fn extend<I>(&mut self, iter: I) where I: IntoIterator<Item=V> {
-        // TODO: maybe use size hint here
-        for i in iter {
+    fn extend<I>(&mut self, iter: I)
+    where
+        I: IntoIterator<Item = V>,
+    {
+        let it = iter.into_iter();
+        
+        reasonable_reserve(&mut self.data, it.size_hint());
+        
+        for i in it {
             self.insert(i);
         }
     }
 }
-impl<'a, V> Extend<&'a V> for CompactMap<V> where V: Copy {
+impl<'a, V> Extend<&'a V> for CompactMap<V>
+where
+    V: Copy,
+{
     #[allow(map_clone)]
-    fn extend<I>(&mut self, iter: I) where I: IntoIterator<Item=&'a V> {
-       self.extend(iter.into_iter().map(|&value| value));
+    fn extend<I>(&mut self, iter: I)
+    where
+        I: IntoIterator<Item = &'a V>,
+    {
+        self.extend(iter.into_iter().map(|&value| value));
     }
 }
 
@@ -451,6 +578,7 @@ impl<V: fmt::Debug> fmt::Debug for CompactMap<V> {
     }
 }
 
+
 macro_rules! generate_iterator {
     ($self_:ident, mut) => {
         generate_iterator!($self_ ; & mut Entry::Occupied(ref mut x), x);
@@ -477,57 +605,72 @@ macro_rules! generate_iterator {
 }
 
 /// An iterator over the key-value pairs of a map.
-pub struct Iter<'a, V : 'a> {
+pub struct Iter<'a, V: 'a> {
     iter: slice::Iter<'a, Entry<V>>,
-    counter : usize,
+    counter: usize,
 }
-impl<'a,V> Iterator for Iter<'a,V> {
+// FIXME(#26925) Remove in favor of `#[derive(Clone)]`
+impl<'a, V> Clone for Iter<'a, V> {
+    fn clone(&self) -> Iter<'a, V> {
+        Iter {
+            iter: self.iter.clone(),
+            counter: self.counter,
+        }
+    }
+}
+impl<'a, V> Iterator for Iter<'a, V> {
     type Item = (usize, &'a V);
-    
+
     #[allow(match_ref_pats)]
     fn next(&mut self) -> Option<(usize, &'a V)> {
         generate_iterator!(self, const);
     }
 }
-impl<'a,V> IntoIterator for &'a CompactMap<V> {
+impl<'a, V> IntoIterator for &'a CompactMap<V> {
     type Item = (usize, &'a V);
     type IntoIter = Iter<'a, V>;
     fn into_iter(self) -> Iter<'a, V> {
-        Iter { iter: self.data.iter(), counter: 0 }
+        Iter {
+            iter: self.data.iter(),
+            counter: 0,
+        }
     }
 }
 
 /// An iterator over the key-value pairs of a map, with the
 /// values being mutable.
-pub struct IterMut<'a, V : 'a> {
+pub struct IterMut<'a, V: 'a> {
     iter: slice::IterMut<'a, Entry<V>>,
-    counter : usize,
+    counter: usize,
 }
-impl<'a,V:'a> Iterator for IterMut<'a,V> {
+impl<'a, V: 'a> Iterator for IterMut<'a, V> {
     type Item = (usize, &'a mut V);
-    
-    #[allow(unused_lifetimes,match_ref_pats)]
+
+    #[allow(unused_lifetimes, match_ref_pats)]
     fn next<'b>(&'b mut self) -> Option<(usize, &'a mut V)> {
         generate_iterator!(self, mut);
     }
 }
 
-impl<'a,V:'a> IntoIterator for &'a mut CompactMap<V> {
+impl<'a, V: 'a> IntoIterator for &'a mut CompactMap<V> {
     type Item = (usize, &'a mut V);
     type IntoIter = IterMut<'a, V>;
     fn into_iter(self) -> IterMut<'a, V> {
-        IterMut { iter: self.data.iter_mut(), counter: 0 }
+        IterMut {
+            iter: self.data.iter_mut(),
+            counter: 0,
+        }
     }
 }
 
 /// A consuming iterator over the key-value pairs of a map.
 pub struct IntoIter<V> {
     iter: vec::IntoIter<Entry<V>>,
-    counter : usize,
+    counter: usize,
 }
 impl<V> Iterator for IntoIter<V> {
     type Item = (usize, V);
-    
+
     fn next(&mut self) -> Option<(usize, V)> {
         generate_iterator!(self, plain);
     }
@@ -536,27 +679,105 @@ impl<V> IntoIterator for CompactMap<V> {
     type Item = (usize, V);
     type IntoIter = IntoIter<V>;
     fn into_iter(self) -> IntoIter<V> {
-        IntoIter { iter: self.data.into_iter(), counter: 0 }
+        IntoIter {
+            iter: self.data.into_iter(),
+            counter: 0,
+        }
     }
 }
 
+
+/// An iterator over the keys of a map.
+pub struct Keys<'a, V: 'a> {
+    iter: Iter<'a, V>,
+}
+// FIXME(#26925) Remove in favor of `#[derive(Clone)]`
+impl<'a, V> Clone for Keys<'a, V> {
+    fn clone(&self) -> Keys<'a, V> {
+        Keys { iter: self.iter.clone() }
+    }
+}
+impl<'a, V> Iterator for Keys<'a, V> {
+    type Item = usize;
+
+    fn next(&mut self) -> Option<usize> {
+        self.iter.next().map(|e| e.0)
+    }
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        self.iter.size_hint()
+    }
+}
+
+/// An iterator over the values of a map.
+pub struct Values<'a, V: 'a> {
+    iter: Iter<'a, V>,
+}
+// FIXME(#19839) Remove in favor of `#[derive(Clone)]`
+impl<'a, V> Clone for Values<'a, V> {
+    fn clone(&self) -> Values<'a, V> {
+        Values { iter: self.iter.clone() }
+    }
+}
+impl<'a, V> Iterator for Values<'a, V> {
+    type Item = &'a V;
+
+    fn next(&mut self) -> Option<&'a V> {
+        self.iter.next().map(|e| e.1)
+    }
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        self.iter.size_hint()
+    }
+}
+
+/// An iterator over the values of a map.
+pub struct ValuesMut<'a, V: 'a> {
+    iter_mut: IterMut<'a, V>,
+}
+impl<'a, V> Iterator for ValuesMut<'a, V> {
+    type Item = &'a mut V;
+
+    fn next(&mut self) -> Option<&'a mut V> {
+        self.iter_mut.next().map(|e| e.1)
+    }
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        self.iter_mut.size_hint()
+    }
+}
+
+/// A draining iterator over the key-value pairs of a map.
+pub struct Drain<'a, V: 'a> {
+    iter: std::iter::FilterMap<
+        std::iter::Enumerate<std::vec::Drain<'a, Entry<V>>>,
+        fn((usize, Entry<V>)) -> Option<(usize, V)>
+        >
+}
+
+impl<'a, V> Iterator for Drain<'a, V> {
+    type Item = (usize, V);
+
+    fn next(&mut self) -> Option<(usize, V)> { self.iter.next() }
+    fn size_hint(&self) -> (usize, Option<usize>) { self.iter.size_hint() }
+}
+
+
+
 #[cfg(feature = "serde")]
 mod serdizer {
     extern crate serde;
-    
+
     use super::CompactMap;
     use super::Entry;
-    
-    use std::usize;
+
+    use super::SENTINEL;
     use self::serde::ser::SerializeMap;
-    
-    impl<V:serde::Serialize> serde::Serialize for CompactMap<V> {
-        fn serialize<S : serde::Serializer>(&self, s:S) -> Result<S::Ok, S::Error> {
+
+    impl<V: serde::Serialize> serde::Serialize for CompactMap<V> {
+        fn serialize<S: serde::Serializer>(&self, s: S) -> Result<S::Ok, S::Error> {
             #[cfg(feature = "serde_ser_len")]
             let len = Some(self.len_slow());
             #[cfg(not(feature = "serde_ser_len"))]
             let len = None;
-            
+
             let mut map = s.serialize_map(len)?;
             for (k, v) in self {
                 map.serialize_entry(&k, v)?;
@@ -564,61 +785,63 @@ mod serdizer {
             map.end()
         }
     }
-    
+
     // Deserializer based on https://serde.rs/deserialize-map.html
-    
+
     use std::fmt;
     use std::marker::PhantomData;
-    
+
     use self::serde::de::{Deserialize, Deserializer, Visitor, MapAccess};
-    
+
     struct MyMapVisitor<V> {
-        marker: PhantomData<fn() -> CompactMap<V>>
+        marker: PhantomData<fn() -> CompactMap<V>>,
     }
-    
+
     impl<V> MyMapVisitor<V> {
         fn new() -> Self {
-            MyMapVisitor {
-                marker: PhantomData
-            }
+            MyMapVisitor { marker: PhantomData }
         }
     }
-    
+
     impl<'de, V> Visitor<'de> for MyMapVisitor<V>
-        where V: Deserialize<'de>
+    where
+        V: Deserialize<'de>,
     {
         type Value = CompactMap<V>;
-    
+
         // Format a message stating what data this Visitor expects to receive.
         fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
             formatter.write_str("a map with small nonnegative integer keys")
         }
 
         fn visit_map<M>(self, mut access: M) -> Result<Self::Value, M::Error>
-            where M: MapAccess<'de>
+        where
+            M: MapAccess<'de>,
         {
             let mut map = CompactMap::with_capacity(access.size_hint().unwrap_or(0));
-    
+
             while let Some((key, value)) = access.next_entry()? {
-            
+
                 // because of Vec::resize_default is unstable
                 while map.data.len() <= key {
-                    map.data.push(Entry::Empty(usize::MAX));
+                    map.data.push(Entry::Empty(SENTINEL));
                 }
                 map.data[key] = Entry::Occupied(value);
             }
             map.reindex();
-    
+
             Ok(map)
         }
     }
-    
+
     // This is the trait that informs Serde how to deserialize MyMap.
     impl<'de, V> Deserialize<'de> for CompactMap<V>
-        where V: Deserialize<'de>
+    where
+        V: Deserialize<'de>,
     {
         fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-            where D: Deserializer<'de>
+        where
+            D: Deserializer<'de>,
         {
             // Instantiate our Visitor and ask the Deserializer to drive
             // it over the input data, resulting in an instance of MyMap.
